'       Hurkle
'
'       Programmed by: DarkDread
'
'       March 5th, 1997
'
'       This program plays the game of Hurkle.  The object of the game is to
'       find the Hurkle (a mythical, furry creature, loved by all!!) which
'       is hidden somewhere on a 10x10 grid.
'
'       Variable list:
'
'       BoxClicked -    Used to check if player has clicked on a box.
'       ChoiceMade -    Used to store player's menu choice.
'       EndGame -       Used as true/false to see if user has quit game.
'       GridX -         Used to get the grid x position of the mouse.
'       GridY -         Used to get the grid y position of the mouse.
'       HurkleFound -   Used to check if the Hurkle has been found.
'       HurkleX -       X position of Hurkle on grid.
'       HurkleY -       Y position of Hurkle on grid.
'       Left -          Used to check left mouse button status.
'       Middle -        Used to check middle mouse button status.
'       MouseX -        X position of mouse in pixels.
'       MouseY -        Y position of mouse in pixels.
'       Music -         Determines if music/sound will be played.
'       Right -         Used to check right mouse button status.
'       Tries -         Used to keep track of the number of times player has
'                       NOT found hurkle during a game.
'
'       Blood1% - Blood3% - Used to hold graphic of blood.
'       Blood1Mask% - Blood3Mask% - Used to hold graphic mask of blood.
'       Destroy% -      Used to hold part of Hurkle game title graphic.
'       DestroyMask% -  Used to hold mask of Hurkle title graphic.
'       GreenVal% -     Used to fade in green hues.
'       Grid% -         Used to hold the main game grid.
'       Hurkle1% - Hurkle5% - Used to hold graphics of Hurkle character.
'       Hurkle1Mask% - Hurkle5Mask% - Used to hold masks of Hurkle character.
'       RedVal% -       Used to fade in red hues.
'       Tree1% - Tree2% - Used to hold tree graphics for main grid.
'
'
'       Public domain mouse routines created by: Hardin Brothers
'       Public domain delay routines created by: Steve Halko
'       Public domain sounds created by:         Matt Zuchowski
'                                                unknown author(s)

' The following lines declare the procedures and their arguments.
DECLARE SUB Center (Sentence$, CursorY%)
DECLARE SUB FadeIn (Colour%)
DECLARE SUB HidePal (Colour%)
DECLARE SUB HowClose (X%, Y%)
DECLARE SUB Info ()
DECLARE SUB InitArray ()
DECLARE SUB InitPal ()
DECLARE SUB KillHurkle (YPos%)
DECLARE SUB KillHurkle2 ()
DECLARE SUB LoadPics ()
DECLARE SUB MainGame ()
DECLARE SUB MainMenu ()
DECLARE SUB MainScreen ()

' Mouse procedure declarations.
DECLARE FUNCTION MouseInit% ()
DECLARE SUB MouseCall ()
DECLARE SUB MouseGetPressInfo (LBtn%, RBtn%, MBtn%, Count%, HPosn%, VPosn%)
DECLARE SUB MouseGetReleaseInfo (LBtn%, RBtn%, MBtn%, Count%, HPosn%, VPosn%)
DECLARE SUB MouseGetStatus (LBtn%, RBtn%, MBtn%, HPosn%, VPosn%)
DECLARE SUB MouseHide ()
DECLARE SUB MouseReadCounters (HCount%, VCount%)
DECLARE SUB MouseSetHorizRange (HMin%, HMax%)
DECLARE SUB MouseSetPosn (HPosn%, VPosn%)
DECLARE SUB MouseSetup ()
DECLARE SUB MouseSetVertRange (VMin%, VMax%)
DECLARE SUB MouseShow ()

DECLARE SUB NewSleep (Ticks%)
DECLARE SUB OKBox ()
DECLARE SUB PrintMessage (Text$)
DECLARE FUNCTION ReadTimer& ()
DECLARE SUB ShowGrid ()
DECLARE SUB ShowGun ()

CONST True = -1                         ' Declaration of constants used
CONST False = 0                         ' within the program.
CONST GreenC = 2
CONST MidX = 20
CONST RedC = 4

DIM SHARED Blood1%(201)                 ' Declaration of arrays to hold
DIM SHARED Blood1Mask%(201)             ' graphics within the program and
DIM SHARED Blood2%(201)                 ' grid.
DIM SHARED Blood2Mask%(201)
DIM SHARED Blood3%(201)
DIM SHARED Blood3Mask%(201)
DIM SHARED Destroy%(4986)
DIM SHARED DestroyMask%(4986)
DIM SHARED GreenVal%(10)
DIM SHARED Grid%(0 TO 9, 0 TO 9)
DIM SHARED Hurkle1%(201)
DIM SHARED Hurkle2%(201)
DIM SHARED Hurkle3%(201)
DIM SHARED Hurkle4%(201)
DIM SHARED Hurkle5%(201)
DIM SHARED Hurkle1Mask%(201)
DIM SHARED Hurkle2Mask%(201)
DIM SHARED Hurkle3Mask%(201)
DIM SHARED Hurkle4Mask%(201)
DIM SHARED Hurkle5Mask%(201)
DIM SHARED RedVal%(10)
DIM SHARED Tree1%(201)
DIM SHARED Tree2%(201)

DIM SHARED BoxClicked AS INTEGER        ' Declare all global variables
DIM SHARED ChoiceMade AS INTEGER        ' within the program.
DIM SHARED EndGame AS INTEGER
DIM SHARED GridX AS INTEGER
DIM SHARED GridY AS INTEGER
DIM SHARED HurkleFound AS INTEGER
DIM SHARED HurkleX AS INTEGER
DIM SHARED HurkleY AS INTEGER
DIM SHARED Left AS INTEGER
DIM SHARED Middle AS INTEGER
DIM SHARED MouseX AS INTEGER
DIM SHARED MouseY AS INTEGER
DIM SHARED Music AS INTEGER
DIM SHARED Right AS INTEGER
DIM SHARED Tries AS INTEGER

ON ERROR GOTO ErrorHandler              ' Create an error trap to check for
                                        ' errors.

CLS : WIDTH 40
CALL Center("Seek and Destroy the Hurkle", 10)
CALL Center("Version 1.00", 12)
CALL Center("Hit 'S' for sound, <ENTER> for none.", 23)

Choice$ = LCASE$(INPUT$(1))
IF Choice$ = "s" THEN Music = True ELSE Music = False

SCREEN 13                               ' 320x200x256 colour screen mode.

  OUT &H3C8, 0                          ' Change background colour to a
  OUT &H3C9, 10                         ' dark shade of brown.
  OUT &H3C9, 0
  OUT &H3C9, 0

'       Beginning of mouse initialization code: =================>>>>
                                                                   '>
TYPE MouseRegs                                                     '>
  AX AS INTEGER                                                    '>
  BX AS INTEGER                                                    '>
  CX AS INTEGER                                                    '>
  DX AS INTEGER                                                    '>
  ES AS INTEGER                                                    '>
END TYPE                                                           '>
                                                                   '>
DIM SHARED MReg AS MouseRegs                                       '>
REDIM SHARED MouseRoutine%(0 TO 19)                                '>
DIM SHARED MouseReady%                                             '>
MouseReady% = 0                                                    '>
CALL MouseSetup                                                    '>
MouseReady% = MouseInit                                            '>
                                                                   '>
DEFINT A-Z                                                         '>
                                                                   '>
TYPE TextMask                                                      '>
  Bg AS INTEGER                                                    '>
  Fg AS INTEGER                                                    '>
  Char AS INTEGER                                                  '>
  Mask AS INTEGER                                                  '>
END TYPE                                                           '>
                                                                   '>
'       End of mouse intialization code. ========================>>>>

CALL LoadPics                           ' Call a procedure to load all of
                                        ' the binary graphic files into
                                        ' arrays.

CALL InitPal                            ' Call a procedure to initialize the
                                        ' palette.

CALL HidePal(GreenC)                    ' Call procedure to set all green
                                        ' hues used in fade to black.

DEF SEG = &HA000                        ' Define graphics segment and load
BLOAD "HURKLE1.FSI"                     ' 320x200x256 binary file to the
                                        ' screen.

CALL FadeIn(GreenC)                     ' Call a procedure to fade in the
                                        ' green hues.

FOR X = 299 TO 151 STEP -2              ' This FOR...NEXT loop moves the
  IF HurkleStep = 0 THEN                ' Hurkle graphic to the center of
      PUT (X, 125), Hurkle1%, PSET      ' the screen by placing an image,
      NewSleep (1)                      ' pausing, erasing the old image
      LINE (X, 125)-(X + 20, 145), 0, BF' and putting a new image 2 pixels
      HurkleStep = 1                    ' over.
    ELSEIF HurkleStep = 1 THEN
      PUT (X, 125), Hurkle2%, PSET
      NewSleep (1)
      LINE (X, 125)-(X + 20, 145), 0, BF
      HurkleStep = 0
  END IF
NEXT X

PUT (X, 125), Hurkle5%, PSET            ' Put Hurkle graphic in the middle
                                        ' of the screen.

CALL ShowGun                            ' Call a procedure to scroll a gun
                                        ' barrel onto the screen.
 
  IF Music THEN PLAY "MB O0 L64 AAEE"   ' Make a noise.
 
  OUT &H3C8, 0                          ' Change background colour to bright
  OUT &H3C9, 0                          ' yellow.
  OUT &H3C9, 63
  OUT &H3C9, 63
 
  CALL NewSleep(1)                      ' Call procedure to pause for 1/18th
                                        ' of a second.
 
  OUT &H3C8, 0                          ' Restore background colour to dark
  OUT &H3C9, 10                         ' brown.
  OUT &H3C9, 0
  OUT &H3C9, 0

CALL KillHurkle(125)                    ' Call a procedure which displays an
                                        ' animation of the Hurkle getting
                                        ' killed.

CALL HidePal(RedC)                      ' Call a procedure to change all red
                                        ' hues used in fade to black.

PUT (79, 35), DestroyMask%, AND         ' Put mask of image onto the screen.
PUT (79, 35), Destroy%, XOR             ' Put image onto the screen.

CALL FadeIn(RedC)                       ' Call a procedure to fade in red
                                        ' hues.

' This is the mainline of the program.
DO
  CALL MainMenu                         ' Call a procedure for the main menu.

  SELECT CASE ChoiceMade                ' Check which choice the player made
  CASE IS = 1                           ' in the menu.
   
    CALL InitArray                      ' Fill the grid array with zeros.
    Tries = 0
     
      RANDOMIZE TIMER                   ' Initialize random number seed.
     
      HurkleX = INT(RND * 10)           ' Get X and Y positions for the
      HurkleY = INT(RND * 10)           ' Hurkle and store them in the grid
      Grid(HurkleX, HurkleY) = 2        ' array.
   
    CALL MainScreen                     ' Call a procedure to draw the main
                                        ' Hurkle screen.
   
    CALL MainGame                       ' Call a procedure to play the game
                                        ' of Hurkle!
  CASE IS = 2
    CALL Info                           ' Call a procedure to display the
                                        ' instructions for the user.
  CASE IS = 3
   
    EndGame = True                      ' Make EndGame true.
  END SELECT

LOOP UNTIL EndGame                      ' Keep going through the mainline of
                                        ' the program until user has chosen
                                        ' quit from the menu and EndGame was
                                        ' made to equal true.

CLS
SYSTEM                                  ' Return to the environment. (ie. DOS)

ErrorHandler:                           ' Routine to handle errors.
  CLS
  SCREEN 0: WIDTH 80                    ' Returns to an 80 column text only
                                        ' screen mode.

  SELECT CASE ERR                       ' SELECT CASE to see which error
                                        ' number was generated.
 
  CASE IS = 53                          ' If error number 53 (file not found)
                                        ' was generated then print the
                                        ' following message:
    PRINT "File not found!"
    PRINT "Make sure that you are in the Hurkle directory and try again."
 
  CASE ELSE                             ' If another error was generated then
                                        ' print the following message and
                                        ' error number:
    PRINT "Unknown error!"
    PRINT "Please contact the programmer and report this error."
    PRINT "Error code number:"; ERR
  END SELECT
 
  PRINT                                 ' Pause for two seconds and return
  PRINT "Exiting to system..."          ' to the environment (ie. DOS)
  SLEEP 2

  SYSTEM

'       The data below is a machine language routine to initialize the
'       mouse and was not written by the author of this program.
'       Beginning of ASM routine: ===============================>>>>
MouseInt:                                                          '>
DATA 55          : 'PUSH BP                                         >
DATA 89, E5      : 'MOV  BP, SP                                     >
DATA 56          : 'PUSH SI                                         >
DATA 8B, 76, 06  : 'MOV  SI,[BP+6]                                  >
DATA 8B, 04      : 'MOV  AX,[SI]                                    >
DATA 8B, 5C, 02  : 'MOV  BX,[SI+2]                                  >
DATA 8B, 4C, 04  : 'MOV  CX,[SI+4]                                  >
DATA 8B, 54, 06  : 'MOV  DX,[SI+6]                                  >
DATA 8E, 44, 08  : 'MOV  ES,[SI+8]                                  >
DATA CD, 33      : 'INT  33h                                        >
DATA 8C, 44, 08  : 'MOV  [SI+8],ES                                  >
DATA 89, 54, 06  : 'MOV  [SI+6],DX                                  >
DATA 89, 4C, 04  : 'MOV  [SI+4],CX                                  >
DATA 89, 5C, 02  : 'MOV  [SI+2],BX                                  >
DATA 89, 04      : 'MOV  [SI],AX                                    >
DATA 5E          : 'POP     SI                                      >
DATA 5D          : 'POP  BP                                         >
DATA CB          : 'RETF                                            >
                                                                   '>
'       End of ASM routine. =====================================>>>>

SUB Center (Sentence$, CursorY%)
  Sentence = INT(LEN(Sentence$) / 2)            ' Gets half of the length of
                                                ' the string passed to this
                                                ' sub.
  X = MidX - Sentence                           ' Calculates where the string
                                                ' should start based on the
                                                ' current screen character
                                                ' resolution.
  LOCATE CursorY%, X                            ' Locates the X and Y pos. of
                                                ' where the string should be.
  PRINT Sentence$                               ' Prints the string on the
                                                ' screen.
END SUB

SUB FadeIn (Colour%)
  ' This sub routine will fade in a set of colours (green or red, depending
  ' on which argument is passed.) by sending new values to the palette
  ' registers using the OUT command.  FOR...NEXT loops are used to create a
  ' "fading" effect.  There is also a system dependent pause create using a
  ' FOR...NEXT loop (But this is a bad example of programming so don't make
  ' it a habit.  I just couldn't get the delay routines to work properly with
  ' the OUT commands, oh well.).
  IF Colour% = GreenC THEN
      DO
        Green% = Green% + 1
        FOR ColourNum = 240 TO 249
          FOR Pause = 1 TO 500: NEXT Pause
          IF Green% < GreenVal%(ColourNum - 239) THEN
            OUT &H3C8, ColourNum
            OUT &H3C9, 0
            OUT &H3C9, Green%
            OUT &H3C9, 0
          END IF
        NEXT ColourNum
      LOOP UNTIL Green% > 55
    ELSEIF Colour% = RedC THEN
      DO
        Red% = Red% + 1
        FOR ColourNum = 230 TO 239
          FOR Pause = 1 TO 500: NEXT Pause
          IF Red% < RedVal%(ColourNum - 229) THEN
            OUT &H3C8, ColourNum
            OUT &H3C9, Red%
            OUT &H3C9, 0
            OUT &H3C9, 0
          END IF
        NEXT ColourNum
      LOOP UNTIL Red% > 55
  END IF
END SUB

SUB HidePal (Colour%)
  ' This sub routine sends values of 0 (or null) to the palette registers
  ' using the OUT command.  This sets the palette attributes we'll be using
  ' in the fades to black.  They are set using FOR...NEXT loops.
  IF Colour% = GreenC THEN
      FOR ColourNum = 240 TO 249
          OUT &H3C8, ColourNum
          OUT &H3C9, 0
          OUT &H3C9, 0
          OUT &H3C9, 0
      NEXT ColourNum
    ELSEIF Colour% = RedC THEN
      FOR ColourNum = 230 TO 239
          OUT &H3C8, ColourNum
          OUT &H3C9, 0
          OUT &H3C9, 0
          OUT &H3C9, 0
      NEXT ColourNum
  END IF
END SUB

SUB HowClose (X, Y)
  ' This sub routine checks what the last mouse click on the grid was and
  ' how close it was to the location of the Hurkle.  It then gives this
  ' information to the user.
  IF X > HurkleX AND Y > HurkleY THEN
      IF X - HurkleX < 3 AND Y - HurkleY < 3 THEN
          Message$ = "Very near"
        ELSEIF X - HurkleX < 5 AND Y - HurkleY < 5 THEN
          Message$ = "  Near   "
        ELSEIF X - HurkleX < 7 AND Y - HurkleY < 7 THEN
          Message$ = "   Far   "
        ELSE
          Message$ = "Very far "
      END IF
    ELSEIF X > HurkleX AND Y < HurkleY THEN
      IF X - HurkleX < 3 AND HurkleY - Y < 3 THEN
          Message$ = "Very near"
        ELSEIF X - HurkleX < 5 AND HurkleY - Y < 5 THEN
          Message$ = "  Near   "
        ELSEIF X - HurkleX < 7 AND HurkleY - Y < 7 THEN
          Message$ = "   Far   "
        ELSE
          Message$ = "Very far "
      END IF
    ELSEIF X < HurkleX AND Y > HurkleY THEN
      IF HurkleX - X < 3 AND Y - HurkleY < 3 THEN
          Message$ = "Very near"
        ELSEIF HurkleX - X < 5 AND Y - HurkleY < 5 THEN
          Message$ = "  Near   "
        ELSEIF HurkleX - X < 7 AND Y - HurkleY < 7 THEN
          Message$ = "   Far   "
        ELSE
          Message$ = "Very far "
      END IF
    ELSEIF X < HurkleX AND Y < HurkleY THEN
      IF HurkleX - X < 3 AND HurkleY - Y < 3 THEN
          Message$ = "Very near"
        ELSEIF HurkleX - X < 5 AND HurkleY - Y < 5 THEN
          Message$ = "  Near   "
        ELSEIF HurkleX - X < 7 AND HurkleY - Y < 7 THEN
          Message$ = "   Far   "
        ELSE
          Message$ = "Very far "
      END IF
  END IF
  CALL PrintMessage(Message$)
END SUB

SUB Info
  ' This sub routine uses the Center procedure with arguments to print user
  ' information onto the screen.  It then calles a sub routine called OKBox
  ' which draws a tiny OK box on the screen which the user must click to
  ' continue.  This is a way of pausing whatever is on the screen so that the
  ' user may read/see it.
  BoxClicked = False
  CLS
  CALL Center("Info", 1)
  CALL Center("Welcome  to Hurkle.  This is a game for", 3)
  CALL Center("one player.  The object of Hurkle is to", 4)
  CALL Center("find  the  Hurkle  (a  mythical,  furry", 5)
  CALL Center("creature, loved  by  all.) on  a  10x10", 6)
  CALL Center("grid.                                  ", 7)
  CALL Center("This is accomplished by using the mouse", 9)
  CALL Center("and left-clicking on  any tree which is", 10)
  CALL Center("on the grid.  If the Hurkle happens to ", 11)
  CALL Center("be  hiding  behind that  tree then  the", 12)
  CALL Center("game is  won.  If  there is  no Hurkle,", 13)
  CALL Center("the tree  is chopped  down and  another", 14)
  CALL Center("tree is selected.                      ", 15)
  CALL Center("This process continues until the Hurkle", 16)
  CALL Center("is found.  The quicker  the  Hurkle  is", 17)
  CALL Center("found,  the  higher the  player's score", 18)
  CALL Center("will  be.  To  quit at any  time,  just", 19)
  CALL Center("left-click on the END button.          ", 20)
  CALL Center("Please click on the box to continue.   ", 21)
  CALL OKBox
END SUB

SUB InitArray
  ' This sub fills the Grid array with zero (or null) values before a new
  ' game is started to ensure that the game screen will always be fresh.
  FOR X = 0 TO 9
    FOR Y = 0 TO 9
      Grid(X, Y) = 0
    NEXT Y
  NEXT X
END SUB

SUB InitPal
  ' This sub routine sends values to the palette registers using the OUT
  ' command to initialize the green and red hues which are used in the fade
  ' routines.  FOR...NEXT loops are used to accomplish this.
  FOR ColourNum = 230 TO 239
    Red% = Red% + 5
      OUT &H3C8, ColourNum
      OUT &H3C9, Red%
      OUT &H3C9, 0
      OUT &H3C9, 0
    RedVal%(ColourNum - 229) = Red%
  NEXT ColourNum
  FOR ColourNum = 240 TO 249
    Green% = Green% + 5
      OUT &H3C8, ColourNum
      OUT &H3C9, 0
      OUT &H3C9, Green%
      OUT &H3C9, 0
    GreenVal%(ColourNum - 239) = Green%
  NEXT ColourNum
END SUB

SUB KillHurkle (YPos%)
  ' This sub puts masks and images on the screen, pausing for 1/18th of a
  ' second between images to create an animation of the Hurkle getting
  ' killed (or splattered, if you prefer.).  The masks are used so that any
  ' background is not destroyed when the new image is put over top of it.
  PUT (150, YPos%), Blood1Mask%, AND
  PUT (150, YPos%), Blood1%, XOR
  NewSleep (1)
  PUT (150, YPos%), Blood2Mask%, AND
  PUT (150, YPos%), Blood2%, XOR
  NewSleep (1)
  PUT (150, YPos%), Blood3Mask%, AND
  PUT (150, YPos%), Blood3%, XOR
  NewSleep (1)
  PUT (150, YPos%), Blood2%, PSET
  NewSleep (1)
  PUT (150, YPos%), Blood1%, PSET
END SUB

SUB KillHurkle2
  ' This sub routine creates an animation which moves the Hurkle to the XPos
  ' middle of the screen from wherever it is hiding.  It then calls the gun
  ' animation and then the original KillHurkle sub with the Hurkle's Y pixel
  ' position passed as an argument.
  ' This sub also uses the play command to play some fast trills when the
  ' Hurkle is found and tries to make it's escape.  It then plays the death
  ' march when the Hurkle is killed (or bludgeoned, if you prefer.).
  IF Music THEN PLAY "t255o1l15deffedo2deffedo3deffedo4deffedo5deffed"
  IF HurkleX > 7 THEN
      FOR X = HurkleX * 20 TO 151 STEP -4
        IF HurkleStep = 0 THEN
            PUT (X, HurkleY * 20), Hurkle1Mask%, AND
            PUT (X, HurkleY * 20), Hurkle1%, XOR
            NewSleep (1)
            FOR TreeX = 0 TO 9
              IF Grid(TreeX, HurkleY) = 0 OR Grid(TreeX, TreeY) = 2 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree1%, PSET
                ELSEIF Grid(TreeX, HurkleY) = 1 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree2%, PSET
              END IF
            NEXT TreeX
            HurkleStep = 1
          ELSEIF HurkleStep = 1 THEN
            PUT (X, HurkleY * 20), Hurkle2Mask%, AND
            PUT (X, HurkleY * 20), Hurkle2%, XOR
            NewSleep (1)
            FOR TreeX = 0 TO 9
              IF Grid(TreeX, HurkleY) = 0 OR Grid(TreeX, TreeY) = 2 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree1%, PSET
                ELSEIF Grid(TreeX, HurkleY) = 1 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree2%, PSET
              END IF
            NEXT TreeX
            HurkleStep = 0
        END IF
      NEXT X
    ELSEIF HurkleX < 8 THEN
      FOR X = HurkleX * 20 TO 151 STEP 4
        IF HurkleStep = 0 THEN
            PUT (X, HurkleY * 20), Hurkle3Mask%, AND
            PUT (X, HurkleY * 20), Hurkle3%, XOR
            NewSleep (1)
            FOR TreeX = 0 TO 9
              IF Grid(TreeX, HurkleY) = 0 OR Grid(TreeX, TreeY) = 2 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree1%, PSET
                ELSEIF Grid(TreeX, HurkleY) = 1 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree2%, PSET
              END IF
            NEXT TreeX
            HurkleStep = 1
          ELSEIF HurkleStep = 1 THEN
            PUT (X, HurkleY * 20), Hurkle4Mask%, AND
            PUT (X, HurkleY * 20), Hurkle4%, XOR
            NewSleep (1)
            FOR TreeX = 0 TO 9
              IF Grid(TreeX, HurkleY) = 0 OR Grid(TreeX, TreeY) = 2 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree1%, PSET
                ELSEIF Grid(TreeX, HurkleY) = 1 THEN
                  PUT (TreeX * 20, HurkleY * 20), Tree2%, PSET
              END IF
            NEXT TreeX
            HurkleStep = 0
        END IF
      NEXT X
  END IF
 
  PUT (X, HurkleY * 20), Hurkle5Mask%, AND
  PUT (X, HurkleY * 20), Hurkle5%, XOR

  CALL ShowGun

    IF Music THEN PLAY "MB O0 L64 AAEE"
    OUT &H3C8, 0
    OUT &H3C9, 0
    OUT &H3C9, 63
    OUT &H3C9, 63
    CALL NewSleep(1)
    OUT &H3C8, 0
    OUT &H3C9, 10
    OUT &H3C9, 0
    OUT &H3C9, 0
 
  CALL KillHurkle(HurkleY * 20)
  IF Music THEN PLAY "l8t200mlo1c..p16c.p32cp32c.p8e..dp32d.cp32c.o0bo1c..mn"
END SUB

SUB LoadPics
  ' This sub routine loads the binary graphic files into arrays by specifing
  ' the segment to load to and the beginning of the array where loading
  ' begins.  If does this until all arrays are filled with their appropriate
  ' graphics.
  DEF SEG = VARSEG(Destroy%(0)): BLOAD "HURKLE1.BSG", VARPTR(Destroy%(0))
  DEF SEG = VARSEG(DestroyMask%(0)): BLOAD "HURKLE2.BSG", VARPTR(DestroyMask%(0))
  DEF SEG = VARSEG(Tree1%(0)): BLOAD "TREE1.TIL", VARPTR(Tree1%(0))
  DEF SEG = VARSEG(Tree2%(0)): BLOAD "TREE2.TIL", VARPTR(Tree2%(0))
  DEF SEG = VARSEG(Hurkle1%(0)): BLOAD "HURKLE1.TIL", VARPTR(Hurkle1%(0))
  DEF SEG = VARSEG(Hurkle2%(0)): BLOAD "HURKLE2.TIL", VARPTR(Hurkle2%(0))
  DEF SEG = VARSEG(Hurkle3%(0)): BLOAD "HURKLE3.TIL", VARPTR(Hurkle3%(0))
  DEF SEG = VARSEG(Hurkle4%(0)): BLOAD "HURKLE4.TIL", VARPTR(Hurkle4%(0))
  DEF SEG = VARSEG(Hurkle5%(0)): BLOAD "HURKLE5.TIL", VARPTR(Hurkle5%(0))
  DEF SEG = VARSEG(Hurkle1Mask%(0)): BLOAD "HURKLE1.MSK", VARPTR(Hurkle1Mask%(0))
  DEF SEG = VARSEG(Hurkle2Mask%(0)): BLOAD "HURKLE2.MSK", VARPTR(Hurkle2Mask%(0))
  DEF SEG = VARSEG(Hurkle3Mask%(0)): BLOAD "HURKLE3.MSK", VARPTR(Hurkle3Mask%(0))
  DEF SEG = VARSEG(Hurkle4Mask%(0)): BLOAD "HURKLE4.MSK", VARPTR(Hurkle4Mask%(0))
  DEF SEG = VARSEG(Hurkle5Mask%(0)): BLOAD "HURKLE5.MSK", VARPTR(Hurkle5Mask%(0))
  DEF SEG = VARSEG(Blood1%(0)): BLOAD "BLOOD1.TIL", VARPTR(Blood1%(0))
  DEF SEG = VARSEG(Blood1Mask%(0)): BLOAD "BLOOD1.MSK", VARPTR(Blood1Mask%(0))
  DEF SEG = VARSEG(Blood2%(0)): BLOAD "BLOOD2.TIL", VARPTR(Blood2%(0))
  DEF SEG = VARSEG(Blood2Mask%(0)): BLOAD "BLOOD2.MSK", VARPTR(Blood2Mask%(0))
  DEF SEG = VARSEG(Blood3%(0)): BLOAD "BLOOD3.TIL", VARPTR(Blood3%(0))
  DEF SEG = VARSEG(Blood3Mask%(0)): BLOAD "BLOOD3.MSK", VARPTR(Blood3Mask%(0))
END SUB

SUB MainGame
  ' This sub routine playes the game of Hurkle.  Check the comments which
  ' are within this sub for a better explanation of what is going on.
 
  CALL MouseShow                        ' Call a procedure to show the mouse
                                        ' cursor so the player can see where
                                        ' the rodent is!

  ' This loop is actually the main game procedure which checks where the
  ' mouse is and what, if any, button has been pressed.  It also puts the
  ' appropriate graphics on the game field.  The loop keeps going until
  ' the Hurkle is found or until the player clicks on the END button.
  DO
    CALL MouseGetStatus(Left, Right, Middle, MouseX, MouseY)
    IF Left = 1 THEN
      GridX = INT((MouseX / 2) / 20)
      GridY = INT(MouseY / 20)
      IF GridX < 10 AND GridY < 10 THEN
        IF Grid(GridX, GridY) <> 1 THEN
          IF Grid(GridX, GridY) = 2 THEN HurkleFound = True
          Grid(GridX, GridY) = 1
          Tries = Tries + 1
          IF NOT HurkleFound THEN CALL HowClose(GridX, GridY)
          CALL ShowGrid
        END IF
      END IF
      IF MouseX / 2 > 246 AND MouseX / 2 < 304 AND MouseY > 16 AND MouseY < 37 THEN EndCurrent = True
    END IF
  LOOP UNTIL HurkleFound OR EndCurrent
 
  CALL MouseHide                        ' Call a procedure to hide the mouse
                                        ' cursor.  This is done so that no
                                        ' garbage is left on the screen after
                                        ' graphics commands are used.
 
  IF HurkleFound THEN CALL KillHurkle2  ' Calls a sub to kill the Hurkle if
                                        ' it has been found.
   
    LINE (35, 80)-(284, 120), 0, BF     ' Draws a box around where some
    LINE (35, 80)-(284, 120), 42, B     ' congratulatory text will be placed.
    LINE (34, 79)-(285, 121), 43, B
    LINE (33, 78)-(286, 122), 42, B
    LINE (32, 77)-(287, 123), 41, B
  IF HurkleFound THEN
      LOCATE 12, 6: PRINT "Congratulations! You          "
      LOCATE 13, 6: PRINT "slaughtered the Hurkle in"; Tries
      LOCATE 14, 6: PRINT "Tries.  Your score is:"; 101 - Tries
      LINE (35, 150)-(284, 165), 0, BF
      LINE (35, 150)-(284, 165), 42, B
      LINE (34, 149)-(285, 166), 43, B
      LINE (33, 148)-(286, 167), 42, B
      LINE (32, 147)-(287, 168), 41, B
     
      IF Tries = 1 THEN                 ' This block IF statement determines
                                        ' what the users rank was based on
                                        ' how many tries it took to find the
                                        ' Hurkle.
          Rank$ = "Lord of Death"
        ELSEIF Tries > 1 AND Tries < 10 THEN
          Rank$ = "Ruthless Killer"
        ELSEIF Tries > 9 AND Tries < 20 THEN
          Rank$ = "Mass Slaughterer"
        ELSEIF Tries > 19 AND Tries < 30 THEN
          Rank$ = "Expert Sniper"
        ELSEIF Tries > 29 AND Tries < 40 THEN
          Rank$ = "Top Hunter"
        ELSEIF Tries > 39 AND Tries < 50 THEN
          Rank$ = "Good Hunter"
        ELSEIF Tries > 49 AND Tries < 60 THEN
          Rank$ = "Decent Shot"
        ELSEIF Tries > 59 AND Tries < 70 THEN
          Rank$ = "Could be Better"
        ELSEIF Tries > 69 AND Tries < 80 THEN
          Rank$ = "Save Some Forest"
        ELSEIF Tries > 79 AND Tries < 90 THEN
          Rank$ = "Blind As a Bat"
        ELSEIF Tries > 89 AND Tries < 100 THEN
          Rank$ = "Utterly Hopeless"
        ELSEIF Tries = 100 THEN
          Rank$ = "You Screwed Up"
      END IF
      LOCATE 20, 6: PRINT "Ranking: "; Rank$
      CALL OKBox                        ' Calls a procedure to display the OK
                                        ' button and wait for user input.
   
    ELSEIF EndCurrent THEN              ' This prints the approprite message
                                        ' if the user gave up without finding
                                        ' the Hurkle.
      LOCATE 12, 6: PRINT "You gave up! The Hurkle got to"
      LOCATE 13, 6: PRINT "live the rest of its life in  "
      LOCATE 14, 6: PRINT "peace...  That really sucks!  "
     
      CALL OKBox                        ' Calls a procedure to display the OK
                                        ' button and wait for user input.
  END IF
 
  HurkleFound = False                   ' Main game variable values are
  EndCurrent = False                    ' returned to zero (or null) so that
                                        ' the next game can start fresh.
END SUB

SUB MainMenu
  ' This sub routine creates a mouse driven main menu with three selections:
  ' "Start", "Information", and "Quit".  The choices are selected using the
  ' mouse and left mouse button.  If the user scrolls over a choice a box is
  ' drawn to highlight that choice.  Once a choice is made, that choice is
  ' entered into the ChoiceMade variable and passed to the mainline where it
  ' is processed.
  CLS
  ChoiceMade = 0
  DEF SEG = &HA000
  BLOAD "HURKLE1.FSI"
  PUT (79, 35), DestroyMask%, AND
  PUT (79, 35), Destroy%, XOR
  LOCATE 17, 19: PRINT "Start"
  LOCATE 19, 16: PRINT "Information"
  LOCATE 21, 20: PRINT "End"
  CALL MouseShow
  DO
    CALL MouseGetStatus(Left, Right, Middle, MouseX, MouseY)
    IF (MouseX / 2) > 99 AND (MouseX / 2) < 226 THEN
      IF MouseY > 124 AND MouseY < 140 THEN
          CALL MouseHide
          LINE (100, 125)-(225, 139), 15, B
          LINE (100, 141)-(225, 155), 0, B
          LINE (100, 157)-(225, 171), 0, B
          IF Left = 1 THEN ChoiceMade = 1
          CALL MouseShow
        ELSEIF MouseY > 140 AND MouseY < 156 THEN
          CALL MouseHide
          LINE (100, 125)-(225, 139), 0, B
          LINE (100, 141)-(225, 155), 15, B
          LINE (100, 157)-(225, 171), 0, B
          IF Left = 1 THEN ChoiceMade = 2
          CALL MouseShow
        ELSEIF MouseY > 156 AND MouseY < 172 THEN
          CALL MouseHide
          LINE (100, 125)-(225, 139), 0, B
          LINE (100, 141)-(225, 155), 0, B
          LINE (100, 157)-(225, 171), 15, B
          IF Left = 1 THEN ChoiceMade = 3
          CALL MouseShow
      END IF
    END IF
  LOOP UNTIL ChoiceMade
  CALL MouseHide
END SUB

SUB MainScreen
  ' This sub routine draws the Hurkle playing field.  It creates a 10x10 grid
  ' of 100 trees and puts the END button in the top right corner.
  CLS
  FOR Y = 0 TO 9
    FOR X = 0 TO 9
      PUT (X * 20, Y * 20), Tree1, PSET
    NEXT X
  NEXT Y
  LOCATE 4, 34: PRINT "END"
  LINE (250, 20)-(300, 34), 42, B
  LINE (249, 19)-(301, 35), 43, B
  LINE (248, 18)-(302, 36), 42, B
  LINE (247, 17)-(303, 37), 41, B
END SUB

SUB MouseCall
'*
'* Call mouse using current values in MReg registers
'*
  DEF SEG = VARSEG(MouseRoutine%(0))
  Addr% = VARPTR(MouseRoutine%(0))
  CALL Absolute(MReg, Addr%)
  DEF SEG
END SUB

SUB MouseGetPressInfo (LBtn%, RBtn%, MBtn%, Count%, HPosn%, VPosn%)
'*
'* Gets info about last mouse click
'* Set LBtn%, RBtn%, or MBtn% <> 0 for button to query
'* LBtn%, RBtn%, or MBtn% return 1 to show which
'* button is currently down or 0 to show which button
'* is currently up
'* Count% has count of clicks since last call
'* HPosn% and VPosn% are horizontal and vertical position
'* of cursor at last click
'*
  IF MouseReady% THEN
    IF (LBtn% OR RBtn% OR MBtn%) <> 0 THEN
      MReg.AX = 5
      IF LBtn% THEN MReg.BX = 0
      IF RBtn% THEN MReg.BX = 1
      IF MBtn% THEN MReg.BX = 2
      MouseCall
      LBtn% = MReg.AX AND 1
      RBtn% = (MReg.AX AND 2) \ 2
      MBtn% = (MReg.AX AND 4) \ 4
      Count% = MReg.BX
      HPosn% = MReg.CX
      VPosn% = MReg.DX
    ELSE
      CALL MouseGetStatus(LBtn%, RBtn%, MBtn%, HPosn%, VPosn%)
      Count% = 0
    END IF
  ELSE
    LBtn% = 0: RBtn% = 0: MBtn% = 0
    Count% = 0
    HPosn% = -1: VPosn% = -1
  END IF
END SUB

SUB MouseGetReleaseInfo (LBtn%, RBtn%, MBtn%, Count%, HPosn%, VPosn%)
'*
'* Gets info about last mouse release
'* Set LBtn%, RBtn%, or MBtn% <> 0 for button query
'* LBtn%, RBtn%, or MBtn% return 1 to show which
'* button is currently down or 0 to show which
'* button is currently up
'* Count% has count of releases since last call
'* HPosn% and VPosn% are horizontal and vertical position
'* of cursor at last release
'*
  IF MouseReady% THEN
    IF (LBtn% OR RBtn% OR MBtn%) <> 0 THEN
      MReg.AX = 6
      IF LBtn% THEN MReg.BX = 0
      IF RBtn% THEN MReg.BX = 1
      IF MBtn% THEN MReg.BX = 2
      MouseCall
      LBtn% = MReg.AX AND 1
      RBtn% = (MReg.AX AND 2) \ 2
      MBtn% = (MReg.AX AND 4) \ 4
      Count% = MReg.BX
      HPosn% = MReg.CX
      VPosn% = MReg.DX
    ELSE
      CALL MouseGetStatus(LBtn%, RBtn%, MBtn%, HPosn%, VPosn%)
      Count% = 0
    END IF
  ELSE
    LBtn% = 0: RBtn% = 0: MBtn% = 0
    Count% = 0
    HPosn% = -1: VPosn% = -1
  END IF
END SUB

SUB MouseGetStatus (LBtn%, RBtn%, MBtn%, HPosn%, VPosn%)
'*
'* Gets current mouse status
'* LBtn%, RBtn%, and MBtn% return
'* 1 if button is down or 0 if button is up
'* HPosn% returns horizontal position
'* VPosn% returns vertical position
'*
  IF MouseReady% THEN
    MReg.AX = 3
    MouseCall
    LBtn% = MReg.BX AND 1
    RBtn% = (MReg.BX AND 2) \ 2
    MBtn% = (MReg.BX AND 4) \ 4
    HPosn% = MReg.CX
    VPosn% = MReg.DX
  ELSE
    LBtn% = 0: RBtn% = 0: MBtn% = 0
    HPosn% = -1: VPosn% = -1
  END IF
END SUB

SUB MouseHide
'*
'* Hide the mouse cursor
'* Decrement the mouse cursor flag
'* Cursor hidden if flag <> 0
'* MouseInit sets flag to -1
'*
  IF MouseReady% THEN
    MReg.AX = 2
    MouseCall
  END IF
END SUB

FUNCTION MouseInit%
'*
'* Initialize mouse and return
'* number of buttons
'* Return 0 if mouse driver
'* unavailable or mouse not
'* present
'*
  DEF SEG = 0
  Sum% = 0
  FOR I% = &H33 * 4 TO &H33 * 4 + 3
    Sum% = Sum% + PEEK(I%)
  NEXT I%
  IF Sum% = 0 THEN
    MouseInit% = 0
    EXIT FUNCTION
  END IF
  MReg.AX = 0
  MouseCall
  IF MReg.AX = 0 THEN
    MouseInit% = 0
  ELSE
    MouseInit% = MReg.BX
  END IF
END FUNCTION

SUB MouseReadCounters (HCount%, VCount%)
'*
'* Returns mouse movement relative to
'* the last time this function was called
'* Measurements are in "mickeys". By default,
'* 1 mickey = 1 pixel horizontally and
'* 2 mickeys = 1 pixel vertically
'*
  IF MouseReady% THEN
    MReg.AX = 11
    MouseCall
    HCount% = MReg.CX
    VCount% = MReg.DX
  END IF
END SUB

SUB MouseSetHorizRange (HMin%, HMax%)
'*
'* Sets the minimum and maximum horizontal
'* range of the mouse cursor.
'* Moves the cursor inside the range if it's
'* outside.
'*
  IF MouseReady% THEN
    MReg.AX = 7
    MReg.CX = HMin%
    MReg.DX = HMax%
    MouseCall
  END IF
END SUB

SUB MouseSetPosn (HPosn%, VPosn%)
'*
'* Sets mouse cursor to HPosn and VPosn
'*
  IF MouseReady% THEN
    MReg.AX = 4
    MReg.CX = HPosn%
    MReg.DX = VPosn%
    MouseCall
  END IF
END SUB

SUB MouseSetup
'*
'* Setup to use mouse
'* MUST be called before
'* MouseInit or any other
'* mouse calls
'*
  RESTORE MouseInt
  DEF SEG = VARSEG(MouseRoutine%(0))
  Addr% = VARPTR(MouseRoutine%(0))
  FOR I = 0 TO 39
    READ a$
    POKE Addr% + I, VAL("&H" + a$)
  NEXT I
  IF a$ <> "CB" THEN ERROR 255
  DEF SEG
END SUB

SUB MouseSetVertRange (VMin%, VMax%)
'*
'* Sets the minimum and maximum vertical
'* range of the mouse cursor
'* Moves the cursor inside the range if it's
'* outside
  IF MouseReady% THEN
    MReg.AX = 8
    MReg.CX = VMin%
    MReg.DX = VMax%
    MouseCall
  END IF
END SUB

SUB MouseShow
'*
'* Display the mouse cursor
'* Increments mouse cursor flag
'* Cursor is displayed if flag is 0
'* MouseInit sets flag to -1
'*
  IF MouseReady% THEN
    MReg.AX = 1
    MouseCall
  END IF
END SUB

SUB NewSleep (Ticks%)
  'This function is a replacement for SLEEP.  It takes the number
  'of system clock ticks as an argument, which gives you much
  'finer control than QB's SLEEP.  To emulate SLEEP with no arguments,
  'use NewSleep(0).  Unlike SLEEP, key presses are cleared from the
  'keyboard buffer.

    StopTime& = ReadTimer& + Ticks%
    DO UNTIL ((ReadTimer& > StopTime&) AND Ticks%) OR LEN(INKEY$)
    LOOP

END SUB

SUB OKBox
  ' This sub routine draws a box with the string "OK" written inside.  It is
  ' used as a pause when the user is reading/viewing the screen.  To exit,
  ' the user must press the left mouse button while the mouse cursor is on
  ' the box.
  BoxClicked = False
  LINE (125, 173)-(178, 185), 0, BF
  CALL Center("OK", 23)
  LINE (125, 173)-(178, 185), 42, B
  LINE (124, 172)-(179, 186), 43, B
  LINE (123, 171)-(180, 187), 42, B
  LINE (123, 170)-(181, 188), 41, B
  CALL MouseShow
  DO
    CALL MouseGetStatus(Left, Right, Middle, MouseX, MouseY)
    IF MouseX / 2 > 122 AND MouseX / 2 < 182 AND MouseY > 169 AND MouseY < 189 THEN
      IF Left = 1 THEN BoxClicked = True
    END IF
  LOOP UNTIL BoxClicked
  CALL MouseHide
END SUB

SUB PrintMessage (Text$)
  LOCATE 9, 31: PRINT Text$
  LINE (235, 60)-(315, 74), 42, B
  LINE (234, 59)-(316, 75), 43, B
  LINE (233, 58)-(317, 76), 42, B
  LINE (232, 57)-(318, 77), 41, B
END SUB

FUNCTION ReadTimer& STATIC
  '[]=============================================================[]
  '[]   Returns the number of clock ticks since midnight
  '[]   without invoking FP emulator like TIMER does
  '[]=============================================================[]

     DEF SEG = &H40
     Lo& = PEEK(&H6C) + 256& * PEEK(&H6D)
     Hi& = PEEK(&H6E) + 256& * PEEK(&H6F)

     ReadTimer& = (65536 * Hi&) + Lo&
END FUNCTION

SUB ShowGrid
  ' This sub routine is used within the main game to refresh the 10x10 grid
  ' whenver the user clicks on a tree in the grid and "chops it down".
  CALL MouseHide
  FOR Y = 0 TO 9
    FOR X = 0 TO 9
      IF Grid%(X, Y) = 0 OR Grid%(X, Y) = 2 THEN
          PUT (X * 20, Y * 20), Tree1%, PSET
        ELSEIF Grid%(X, Y) = 1 THEN
          PUT (X * 20, Y * 20), Tree2%, PSET
      END IF
    NEXT X
  NEXT Y
  CALL MouseShow
END SUB

SUB ShowGun
  ' This sub routine creates an animation of a gun barrel scrolling from the
  ' bottom of the screen by using the LINE command in a FOR...NEXT loop.
  FOR Y = 199 TO 160 STEP -3
    LINE (159, 199)-(159, Y), 16
   
    LINE (158, 199)-(157, Y), 19, B
    LINE (156, 199)-(156, Y), 22
    LINE (155, 199)-(154, Y), 19, B

    LINE (160, 199)-(161, Y), 19, B
    LINE (162, 199)-(162, Y), 22
    LINE (163, 199)-(164, Y), 19, B
    CALL NewSleep(2)
  NEXT Y
END SUB

